# -*- coding: utf-8 -*-
"""
Created on Mon May 15 14:31:32 2023

@author: lucas
"""

import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
import pandas as pd

inch = 0.39370078740157477

## Define figure presets
mpl.rcdefaults()
mpl.rcParams['figure.dpi'] = 200 
mpl.rcParams['figure.figsize'] = (3.5*inch,2.5*inch) 

mpl.rcParams['axes.linewidth'] = 0.5
mpl.rcParams['lines.linewidth'] = 1
mpl.rcParams["legend.numpoints"] = 1
mpl.rcParams["legend.frameon"] = False
#mpl.rcParams['figure.figsize'] = (4, 3)
mpl.rcParams['font.size'] = 10
mpl.rcParams['font.family'] = 'Arial'

# #Setting finally the Text characters to regular instead of italics.
mpl.rcParams['mathtext.rm'] = 'Arial'                 #'custom''Arial' #custom make mu italic, Arial straight
mpl.rcParams['mathtext.it'] = 'Arial: italic'         #make latex italics unless use \mathrm{}
mpl.rcParams['mathtext.default'] = 'it'#'regular'
mpl.rcParams['xtick.direction'] = 'in'
mpl.rcParams['ytick.direction'] = 'in'
mpl.rcParams['xtick.major.width'] = 0.5
mpl.rcParams['ytick.major.width'] = 0.5
mpl.rcParams['xtick.major.size'] = 3.5 #default 3.5
mpl.rcParams['xtick.minor.size'] = 2 #default 2.0
mpl.rcParams['ytick.major.size'] = 3.5 #default 3.5
mpl.rcParams['ytick.minor.size'] = 2 #default 2.0

mpl.rcParams["errorbar.capsize"]= 3
mpl.rcParams['savefig.bbox'] = 'tight' # to avoid clipping mask cropping the axis label

## Setup figure
fig, ax = plt.subplots(figsize=(4,4))

def plt_count_Ramanshift(fig, ax):
    ## Import Raman spectrum 
    ## Ge/SiGe sub3
    path = r"W:\staff-groups\tnw\ns\qt\ScappucciLab\0_Group members\Lucas\Publications\APL_2023\DataRepository\Fig2\panel_a\Raman_spectrum.txt"
    
    df = pd.read_csv(path, sep='\t', header=None)
    
    data = df.to_numpy()
    shift = data[0,2:]
    count = data[:,2:]/1e3
    
    ax.plot(shift, count[221,:], 'k', lw=1)
    ax.text(0, 0, '$\omega_\mathdefault{SiGe}$', fontsize=8)
    plt.text(1, 1, '$\omega_{\mathdefault{Ge}}$', fontsize=8)
    ax.set_xlim([270, 320])
    ax.set_ylim([0,1.5])
    ax.set_xlabel('Raman shift ($\mathdefault{cm}^{\mathdefault{-1}}$)')
    ax.set_ylabel('Intensity ($\\times\mathdefault{10^3}$ counts)', labelpad=2)
    ax.tick_params(direction='in')
    
plt_count_Ramanshift(fig, ax)
# fig.tight_layout()




