# -*- coding: utf-8 -*-
"""
Created on Mon May 15 14:43:36 2023

@author: lucas
"""

import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
import pandas as pd
from scipy.optimize import curve_fit

inch = 0.39370078740157477

## Define figure presets
mpl.rcdefaults()
mpl.rcParams['figure.dpi'] = 200 
mpl.rcParams['figure.figsize'] = (3.5*inch,2.5*inch) 

mpl.rcParams['axes.linewidth'] = 0.5
mpl.rcParams['lines.linewidth'] = 1
mpl.rcParams["legend.numpoints"] = 1
mpl.rcParams["legend.frameon"] = False
#mpl.rcParams['figure.figsize'] = (4, 3)
mpl.rcParams['font.size'] = 10
mpl.rcParams['font.family'] = 'Arial'

# #Setting finally the Text characters to regular instead of italics.
mpl.rcParams['mathtext.rm'] = 'Arial'                 #'custom''Arial' #custom make mu italic, Arial straight
mpl.rcParams['mathtext.it'] = 'Arial: italic'         #make latex italics unless use \mathrm{}
mpl.rcParams['mathtext.default'] = 'it'#'regular'
mpl.rcParams['xtick.direction'] = 'in'
mpl.rcParams['ytick.direction'] = 'in'
mpl.rcParams['xtick.major.width'] = 0.5
mpl.rcParams['ytick.major.width'] = 0.5
mpl.rcParams['xtick.major.size'] = 3.5 #default 3.5
mpl.rcParams['xtick.minor.size'] = 2 #default 2.0
mpl.rcParams['ytick.major.size'] = 3.5 #default 3.5
mpl.rcParams['ytick.minor.size'] = 2 #default 2.0

mpl.rcParams["errorbar.capsize"]= 3
mpl.rcParams['savefig.bbox'] = 'tight' # to avoid clipping mask cropping the axis label

## Set up figure
fig, ax = plt.subplots(figsize=(4,4))

def linear(x, a, b):
    return a*x+b
 
def plt_GeQWShift_SiGeShift(fig, ax):
    ## GeGe in Ge QW
    path1 = r"W:\staff-groups\tnw\ns\qt\ScappucciLab\0_Group members\Lucas\Publications\APL_2023\DataRepository\Fig2\panel_b\GeGe in Ge QW.txt"
    
    ## GeGe in SiGe
    path2 = r"W:\staff-groups\tnw\ns\qt\ScappucciLab\0_Group members\Lucas\Publications\APL_2023\DataRepository\Fig2\panel_b\GeGe in SiGe.txt"
       
    df1 = pd.read_csv(path1, sep='\t', header=None)
    data1 = df1.to_numpy()  
    raman_shift_Ge = data1[1:,1:] 
    
    df2 = pd.read_csv(path2, sep='\t', header=None)
    data2 = df2.to_numpy()    
    raman_shift_SiGe = data2[1:,1:]
      
    raman_shift_Ge = raman_shift_Ge.reshape(int(21*21))    
    raman_shift_SiGe = raman_shift_SiGe.reshape(int(21*21))    
    ax.plot(raman_shift_SiGe, raman_shift_Ge, 'k.', fillstyle='none', ms=2, label='data')
    popt, pcov = curve_fit(linear, raman_shift_SiGe, raman_shift_Ge)
    x = np.linspace(np.min(raman_shift_SiGe), np.max(raman_shift_SiGe), 100)
    ax.plot(x, linear(x, *popt), '-r', label='fit', lw=1.5)
    ax.set_xlabel('$\omega_{\mathdefault{SiGe}}$ (cm$^{\mathdefault{-1}}$)')
    ax.set_ylabel('$\omega_{\mathdefault{Ge}}$ (cm$^{\mathdefault{-1}}$)')


plt_GeQWShift_SiGeShift(fig, ax)
fig.tight_layout()
