# -*- coding: utf-8 -*-
"""
Created on Mon May 15 14:52:41 2023

@author: lucas
"""

import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
from mpl_toolkits.axes_grid.inset_locator import inset_axes
import pandas as pd
# from scipy.constants import h, k, e, m_e, epsilon_0, pi

inch = 0.39370078740157477

## Define figure presets
mpl.rcdefaults()
mpl.rcParams['figure.dpi'] = 200 
mpl.rcParams['figure.figsize'] = (3.5*inch,2.5*inch) 

mpl.rcParams['axes.linewidth'] = 0.5
mpl.rcParams['lines.linewidth'] = 1
mpl.rcParams["legend.numpoints"] = 1
mpl.rcParams["legend.frameon"] = False
#mpl.rcParams['figure.figsize'] = (4, 3)
mpl.rcParams['font.size'] = 10
mpl.rcParams['font.family'] = 'Arial'

# #Setting finally the Text characters to regular instead of italics.
mpl.rcParams['mathtext.rm'] = 'Arial'                 #'custom''Arial' #custom make mu italic, Arial straight
mpl.rcParams['mathtext.it'] = 'Arial: italic'         #make latex italics unless use \mathrm{}
mpl.rcParams['mathtext.default'] = 'it'#'regular'
mpl.rcParams['xtick.direction'] = 'in'
mpl.rcParams['ytick.direction'] = 'in'
mpl.rcParams['xtick.major.width'] = 0.5
mpl.rcParams['ytick.major.width'] = 0.5
mpl.rcParams['xtick.major.size'] = 3.5 #default 3.5
mpl.rcParams['xtick.minor.size'] = 2 #default 2.0
mpl.rcParams['ytick.major.size'] = 3.5 #default 3.5
mpl.rcParams['ytick.minor.size'] = 2 #default 2.0

mpl.rcParams["errorbar.capsize"]= 3
mpl.rcParams['savefig.bbox'] = 'tight' # to avoid clipping mask cropping the axis label

def RamanShift_to_Strain(x, strainedShift, unstrainedShift):   
    # References: [1] F.Pezzoli et al. "Phonon strain shift coefficients in
    # Si(1-x)Ge(x) alloys" JOURNAL OF APPLIED PHYSICS 103, 093521 (2008)
    #
    # In this script, once you provide the Ge fraction x in
    # Si(1-x)Ge(x), and the corresponding Raman peaks (cm-1) for the strained ws and
    # unstrained w0 material, the strain epsilon is calculated.
    # Provide the Ge fraction in the material of interest as 0<=x<=1 range.
    # Provide the values of strained and unstrained Raman peaks for x = 0 and
    # x = 1 as single values, in Si and Ge, respectively.
    # Provide 3 values array for strained and unstrained Raman peaks for SiGe
    # alloy where 0<x<1. The resultant strain will be averaged considering
    # the three Raman Shift for Ge-Ge / Si-Si / Si-Ge modes.
    # relaxed Ge-Ge from Ge is 299.928cm-1
    # relaxed Si-Si from Si is 520.7cm-1
    
    ## SiGe coeff
    if x==1:
        ws = strainedShift
        w0 = unstrainedShift
        bGe = -440 # +/-8
        
    if x!= 0 and x!=1:
        # Si
        wsSi = strainedShift[0]
        w0Si = unstrainedShift[0]
        # SiGe
        wsSiGe = strainedShift[1]
        w0SiGe = unstrainedShift[1]
        # Ge
        wsGe = strainedShift[2]
        w0Ge = unstrainedShift[2]
    
    if x == 0.82:
        bGe = -460 # +/-40
        bSi = -650 # +/-100 
        bSiGe = -530 # +/-70

    if x == 0.64:
        bGe = -470 # +/-40
        bSi = -650 # +/-100 
        bSiGe = -790 # +/-70

    if x == 0.42:
        bGe = -430 # +/-40
        bSi = -720 # +/-70 
        bSiGe = -580 # +/-40

    if x == 0.20:
        # bGe   = -460 # +/-40
        bSi   = -780 # +/-80 
        # bSiGe = -530 # +/-70

    if x == 0:
        ws = strainedShift
        w0 = unstrainedShift
        bSi = -723 # +/-15
        
    ## Strain
    # epsilon = (ws - w0)/b (- is compressive, + is tensile)
    if x == 1:
        epsilon = ((ws - w0)/bGe) 
    if x == 0:
        epsilon = ((ws - w0)/bSi) 
    if x!=0 and x!=1:
        epsilon1 = ((wsSi - w0Si)/bSi)  
        epsilon2 = ((wsSiGe - w0SiGe)/bSiGe)
        epsilon3 = ((wsGe - w0Ge)/bGe) 
        epsilon = (epsilon1 + epsilon2 + epsilon3)/3
    
    return epsilon

def plt_GeQWstrain(fig, ax, strain):
    strain = strain*1e3
    X = np.linspace(-15, 15, 21)+15 
    Y = np.linspace(-15, 15, 21)+15
    
    im = ax.pcolormesh(X, Y, strain, cmap='magma', rasterized=True)
    
    ax.set_xlabel('$x$ (\u03BCm)')
    ax.set_ylabel('$y$ (\u03BCm)')

    ax.set_xticks([0,10,20,30])
    ax.set_yticks([0,10,20,30])
    
    cax_inset = inset_axes(ax, width='2%', height='30%', loc='upper left',
                           bbox_to_anchor=(1,0.052, 1.5, 0.97), bbox_transform=ax.transAxes,)
    cbar = fig.colorbar(im, ticks=[-6.5,-7], cax=cax_inset, orientation='vertical')

    cbar.ax.tick_params(labelsize=8, width=0)
    cax_inset.yaxis.set(ticks_position='right')
    cax_inset.yaxis.set_tick_params(pad=1)
    cbar.set_label('$\epsilon$ ($\mathdefault{10^{-3}}$)', labelpad=-25, y=-0.7)

## GeGe in Ge QW
path = r"W:\staff-groups\tnw\ns\qt\ScappucciLab\0_Group members\Lucas\Publications\APL_2023\DataRepository\Fig2\panel_c\GeGe in Ge QW.txt"

df = pd.read_csv(path, sep='\t', header=None)
data = df.to_numpy()

raman_shift = data[1:,1:]

## Calculate strain from phonon shift
strain = np.zeros((len(raman_shift), len(raman_shift)))
for i in range(len(strain)):
    for j in range(len(strain)):
        ## GeGe mode
        strain[i,j] = RamanShift_to_Strain(1, raman_shift[i,j], 299.928) 
        
## Setup figure
fig, ax = plt.subplots(figsize = (4,4))
plt_GeQWstrain(fig, ax, strain)




