# -*- coding: utf-8 -*-
"""
Created on Mon May 15 15:11:47 2023

@author: lucas
"""

import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
from scipy.stats import norm

inch = 0.39370078740157477

## Define figure presets
mpl.rcdefaults()
mpl.rcParams['figure.dpi'] = 200 
mpl.rcParams['figure.figsize'] = (3.5*inch,2.5*inch) 

mpl.rcParams['axes.linewidth'] = 0.5
mpl.rcParams['lines.linewidth'] = 1
mpl.rcParams["legend.numpoints"] = 1
mpl.rcParams["legend.frameon"] = False
#mpl.rcParams['figure.figsize'] = (4, 3)
mpl.rcParams['font.size'] = 10
mpl.rcParams['font.family'] = 'Arial'

# #Setting finally the Text characters to regular instead of italics.
mpl.rcParams['mathtext.rm'] = 'Arial'                 #'custom''Arial' #custom make mu italic, Arial straight
mpl.rcParams['mathtext.it'] = 'Arial: italic'         #make latex italics unless use \mathrm{}
mpl.rcParams['mathtext.default'] = 'it'#'regular'
mpl.rcParams['xtick.direction'] = 'in'
mpl.rcParams['ytick.direction'] = 'in'
mpl.rcParams['xtick.major.width'] = 0.5
mpl.rcParams['ytick.major.width'] = 0.5
mpl.rcParams['xtick.major.size'] = 3.5 #default 3.5
mpl.rcParams['xtick.minor.size'] = 2 #default 2.0
mpl.rcParams['ytick.major.size'] = 3.5 #default 3.5
mpl.rcParams['ytick.minor.size'] = 2 #default 2.0

mpl.rcParams["errorbar.capsize"]= 3
mpl.rcParams['savefig.bbox'] = 'tight' # to avoid clipping mask cropping the axis label

## Setup figure
fig, ax = plt.subplots(figsize=(4,4))

def plt_strainHistogram(fig, ax):
    path = r"W:\staff-groups\tnw\ns\qt\ScappucciLab\0_Group members\Lucas\Publications\APL_2023\DataRepository\Fig2\panel_d\abs_strain_GeQW.npy"
    strain = np.load(path)
    strain_zero = (strain - np.mean(strain)) 
    strain_hist = strain_zero.reshape(int(len(strain_zero)*len(strain_zero)))

   
    ax.hist(strain_hist, bins=20, density=1, alpha=1, rasterized=True)
    mu, std = norm.fit(strain_hist)
    x = np.linspace(-0.8, 0.8, 100)
    p = norm.pdf(x, mu, std)
    ax.plot(x, p, 'k', lw=1)
    
    ax.set_xlabel('$\\Delta\epsilon$ ($\mathdefault{10^{-3}}$)', labelpad=-0.5) 
    ax.set_ylabel('Probability density')
    ax.set_xlim([-0.8, 0.8])
    ax.set_xticks([-0.8, 0, .8])
    
plt_strainHistogram(fig, ax)
