from core_tools.data.ds.ds2xarray import ds2xarray
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
from scipy.constants import e
import json
from core_tools.data.ds.ds_hdf5 import load_hdf5_uuid

def linear(x,a,b):
    ## linear function for fitting
    return a*x+b

def get_param_val(ds, param):
    return json.loads(ds.attrs['snapshot'])['station']['parameters'][param]['value']

def getDensity(rho_xy):
    coeff = rho_xy.curvefit(coords='field', func=linear).curvefit_coefficients
    n = 1/(coeff[:,0]*e) / (1e4)
    return n

def getMobility(n, rho_xx):
    mu = (1 / (e*n*rho_xx.sel(field=0, method='nearest'))) 
    return mu

def calc_mob_dens(uuid, aspect_ratio, Vg, Bfield, dsx=None):
    if uuid!=None and dsx==None:
        # dsx = ds2xarray(load_by_uuid(uuid)).dropna('Vg')
        dsx = ds2xarray(load_hdf5_uuid(uuid, path)).dropna('Vg')
        try:
            sampleNos = int(dsx.title.split()[-1]) # Is there something better for this? 
        except: 
            sampleNos = 1
        dsx = dsx.assign_coords({'sample': (sampleNos)})
    elif uuid==None and dsx!=None:
        dsx = dsx.dropna('Vg')
        sampleNos = int(np.array(dsx.sample))
    
    if dsx.Vxx.mean('field').mean('Vg')<0:
        dsx['Vxx'] = -dsx['Vxx']
    
    if dsx.Vxy.mean('Vg')[0] > dsx.Vxy.mean('Vg')[-1]:
        dsx['Vxy'] = -dsx['Vxy']
        
    ## Sort data along Bfield
    dsx = dsx.sortby('field')
    dsx['Isdraw'] = dsx.Iac.sel(field=0, method='nearest')*1e9
    dsx['Vgraw'] = dsx.Vg
        
    ## filter data based on Vg
    dsx = dsx.sel(Vg=Vg)
    
    ## crop data based on Bfield
    dsx = dsx.sel(field=Bfield)
    
    ## get rho_xx and rho_xy from dataset
    dsx['rho_xx'] = dsx.Vxx/dsx.Iac/aspect_ratio
    dsx['rho_xy'] = dsx.Vxy/dsx.Iac
    dsx['sigma_xx'] = 1 / dsx.rho_xx.sel(field=0, method='nearest')
    
    ## get mobility and density
    dsx['n'] = getDensity(dsx['rho_xy']) 
    dsx['mu'] = getMobility(dsx['n'], dsx['rho_xx'])
        
    return dsx

## Load uuids of raw data set and the path to the folder where they are saved in
uuid = [1677098782290750695, 1677100930189750695, 1677103104463750695, 1677105279527750695, 1677107451504750695, 1677109627493750695]
path = r"W:\staff-groups\tnw\ns\qt\ScappucciLab\0_Group members\Lucas\Publications\APL_2023\DataRepository\Fig3\panel_a"

## Extract maximum mobility from raw data
mu_max = []
for i in range(len(uuid)):

    if i==0:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1758, None), Bfield=slice(-0.1, 0.1))
    elif i==1:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1758, None), Bfield=slice(-0.1, 0.1))
    elif i==2:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1760, None), Bfield=slice(-0.1, 0.1))
    elif i==3:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1755, None), Bfield=slice(-0.1, 0.1))
    elif i==4:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1765, None), Bfield=slice(-0.1, 0.1))
    elif i==5:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1760, None), Bfield=slice(-0.1, 0.1))    

    print('mu_max= %0.2f' %dsx.mu.max())    
    mu_max.append(dsx.mu.max())

mu_max = np.array(mu_max)

## Calculate average mobility with std and density for Fig.3a of manuscript
n = []
mu = []
nmax = []
nmin = []
nlen = []
Vg = []

for i in range(len(uuid)):

    if i==0:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1758, None), Bfield=slice(-0.1, 0.1))
    elif i==1:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1758, None), Bfield=slice(-0.1, 0.1))
    elif i==2:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1760, None), Bfield=slice(-0.1, 0.1))
    elif i==3:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1755, None), Bfield=slice(-0.1, 0.1))
    elif i==4:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1765, None), Bfield=slice(-0.1, 0.1))
    elif i==5:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1760, None), Bfield=slice(-0.1, 0.1)) 
    
    n.append(np.array(dsx.n))
    nmin.append(np.array(dsx.n.min()))
    nmax.append(np.array(dsx.n.max()))
    nlen.append(np.array(dsx.n.size))
    mu.append(np.array(dsx.mu))
    Vg.append(np.array(dsx.Vg))
 
max_mindens = np.max(nmin)
min_maxdens = np.min(nmax)
max_len = np.max(nlen)
n_intp = np.linspace(max_mindens, min_maxdens, max_len)
mu_array = np.zeros((len(n_intp), len(mu)))

for i in range(len(mu)):
    mu_array[:,i] = np.interp(n_intp, n[i], mu[i])
   
n_average = n_intp/1e11
mean_mu = np.mean(mu_array, axis=1)/1e5
mu_std = np.std(mu_array, axis=1)/1e5

## Plot average mobility-density curve 
inch = 0.39370078740157477

## Define figure presets
mpl.rcdefaults()
mpl.rcParams['figure.dpi'] = 200 
mpl.rcParams['figure.figsize'] = (3.5*inch,2.5*inch) 

mpl.rcParams['axes.linewidth'] = 0.5
mpl.rcParams['lines.linewidth'] = 1
mpl.rcParams["legend.numpoints"] = 1
mpl.rcParams["legend.frameon"] = False
#mpl.rcParams['figure.figsize'] = (4, 3)
mpl.rcParams['font.size'] = 10
mpl.rcParams['font.family'] = 'Arial'

# #Setting finally the Text characters to regular instead of italics.
mpl.rcParams['mathtext.rm'] = 'Arial'                 #'custom''Arial' #custom make mu italic, Arial straight
mpl.rcParams['mathtext.it'] = 'Arial: italic'         #make latex italics unless use \mathrm{}
mpl.rcParams['mathtext.default'] = 'it' #'default'
mpl.rcParams['xtick.direction'] = 'in'
mpl.rcParams['ytick.direction'] = 'in'
mpl.rcParams['xtick.major.width'] = 0.5
mpl.rcParams['ytick.major.width'] = 0.5
mpl.rcParams['xtick.major.size'] = 3.5 #default 3.5
mpl.rcParams['xtick.minor.size'] = 2 #default 2.0
mpl.rcParams['ytick.major.size'] = 3.5 #default 3.5
mpl.rcParams['ytick.minor.size'] = 2 #default 2.0

mpl.rcParams["errorbar.capsize"]= 3
mpl.rcParams['savefig.bbox'] = 'tight' # to avoid clipping mask cropping the axis label

## Setup figure
fig, ax = plt.subplots(figsize=(4,4))

def plt_mobdens(fig, ax, mean_mu, mu_std, n_average, mu_max):
    mean_mu = mean_mu/10
    mu_std = mu_std/10
    mu_max = mu_max/1e6
    mu_max_mean = np.mean(mu_max)
    mu_max_std = np.std(mu_max)
    
    ax.plot(n_average, mean_mu, 'k')
    ax.fill_between(n_average, mean_mu-mu_std, mean_mu+mu_std, alpha=0.3, color='k', edgecolor=None)
    ax.set_ylabel('$\mu$ ($\mathdefault{10^{6}}$ $\mathdefault{cm}^2/\mathdefault{Vs}$)', labelpad=10) 
    ax.set_xlabel('$p$ ($\mathdefault{10^{11}}$ $\mathdefault{cm}^{-2}$)', labelpad=1)
    ax.text(0.96,2.2, '$\mathdefault{10^{6}}$', fontsize=8)
    ax.set_xticks([0.5, 1.0, 1.5])
    ax.set_yticks([0,1,2,3])
    
    axin = ax.inset_axes([0.55, 0.1, 0.4, 0.5])
    x = np.linspace(0.1, 0.9, len(mu_max))
    axin.plot(x, mu_max, 'kd', ms=2)
    axin.errorbar((x.max()-x.min())/2+x.min(), mu_max_mean, mu_max_std, fmt='ko', ms=3, 
                  ecolor='k', elinewidth=1, capsize=2)
    axin.tick_params(labelsize=8)
    axin.set_ylabel('$\mu_{\mathdefault{max}}$ ($\mathdefault{cm}^2/\mathdefault{Vs}$)', 
                    fontsize=8, labelpad=5, verticalalignment='center') 
    # plt.text(0, 0, '$10^{6}$', fontsize=8)
    axin.set_ylim([3.2, 3.7])
    axin.set_xlim([0,1])
    axin.set_xticks([])
    axin.set_xticklabels([])

plt_mobdens(fig, ax, mean_mu, mu_std, n_average, mu_max)

















