from core_tools.data.ds.ds2xarray import ds2xarray
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
from scipy.constants import e, h
import json
from core_tools.data.ds.ds_hdf5 import load_hdf5_uuid
from scipy.optimize import curve_fit

def linear(x,a,b):
    ## linear function for fitting
    return a*x+b

def get_param_val(ds, param):
    return json.loads(ds.attrs['snapshot'])['station']['parameters'][param]['value']

def getDensity(rho_xy):
    coeff = rho_xy.curvefit(coords='field', func=linear).curvefit_coefficients
    n = 1/(coeff[:,0]*e) / (1e4)
    return n

def getMobility(n, rho_xx):
    mu = (1 / (e*n*rho_xx.sel(field=0, method='nearest'))) 
    return mu

def calc_mob_dens(uuid, aspect_ratio, Vg, Bfield, dsx=None):
    if uuid!=None and dsx==None:
        # dsx = ds2xarray(load_by_uuid(uuid)).dropna('Vg')
        dsx = ds2xarray(load_hdf5_uuid(uuid, path)).dropna('Vg')
        try:
            sampleNos = int(dsx.title.split()[-1]) # Is there something better for this? 
        except: 
            sampleNos = 1
        dsx = dsx.assign_coords({'sample': (sampleNos)})
    elif uuid==None and dsx!=None:
        dsx = dsx.dropna('Vg')
        sampleNos = int(np.array(dsx.sample))
    
    if dsx.Vxx.mean('field').mean('Vg')<0:
        dsx['Vxx'] = -dsx['Vxx']
    
    if dsx.Vxy.mean('Vg')[0] > dsx.Vxy.mean('Vg')[-1]:
        dsx['Vxy'] = -dsx['Vxy']
        
    ## Sort data along Bfield
    dsx = dsx.sortby('field')
    dsx['Isdraw'] = dsx.Iac.sel(field=0, method='nearest')*1e9
    dsx['Vgraw'] = dsx.Vg
        
    ## filter data based on Vg
    dsx = dsx.sel(Vg=Vg)
    
    ## crop data based on Bfield
    dsx = dsx.sel(field=Bfield)
    
    ## get rho_xx and rho_xy from dataset
    dsx['rho_xx'] = dsx.Vxx/dsx.Iac/aspect_ratio
    dsx['rho_xy'] = dsx.Vxy/dsx.Iac
    dsx['sigma_xx'] = 1 / dsx.rho_xx.sel(field=0, method='nearest')
    
    ## get mobility and density
    dsx['n'] = getDensity(dsx['rho_xy']) 
    dsx['mu'] = getMobility(dsx['n'], dsx['rho_xx'])
        
    return dsx

## Load uuids of raw data set and the path to the folder where they are saved in
uuid = [1677098782290750695, 1677100930189750695, 1677103104463750695, 1677105279527750695, 1677107451504750695, 1677109627493750695]
path = r"W:\staff-groups\tnw\ns\qt\ScappucciLab\0_Group members\Lucas\Publications\APL_2023\DataRepository\Fig3\panel_b"

## Find percolation density np from sigma_xx = A(n-np)^p
def percolation_function(n,A,n_p):
    return A*(n-n_p)**1.31
n_p = np.zeros(len(uuid))
       
## Fitting the percolation density for each sample
Vg_min = np.array([-1757, -1759, -1759, -1754, -1765, -1759])
for i in range(len(uuid)):
    Vgmin = Vg_min[i]
    
    if i==0:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(Vgmin, None), Bfield=slice(-0.1, 0.1))
    elif i==1:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(Vgmin, None), Bfield=slice(-0.1, 0.1))
    elif i==2:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(Vgmin, None), Bfield=slice(-0.1, 0.1))
    elif i==3:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(Vgmin, None), Bfield=slice(-0.1, 0.1))
    elif i==4:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(Vgmin, None), Bfield=slice(-0.1, 0.1))
    elif i==5:
        dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(Vgmin, None), Bfield=slice(-0.1, 0.1))    
    
    sigma_xx = dsx.sigma_xx * (h/e**2)
    n = dsx.n/1e11
    
    ## Fit for percolation density based on fixed range 
    n_min = n.min()
    n_fit_range = 0.5
    
    n_min_idx = int(abs(n-n_min).argmin()) + 0
    n_max = n[n_min_idx] + n_fit_range
    n_max_idx = int(abs(n-n_max).argmin())
     
    popt, pcov = curve_fit(percolation_function, n[n_min_idx:n_max_idx], sigma_xx[n_min_idx:n_max_idx], 
                           p0=(0.005,0), bounds=([0,0], [np.inf, n_max]), maxfev=100000) 
    
    n_p[i] = popt[1]

## Plot average mobility-density curve 
inch = 0.39370078740157477

## Define figure presets
mpl.rcdefaults()
mpl.rcParams['figure.dpi'] = 200 
mpl.rcParams['figure.figsize'] = (3.5*inch,2.5*inch) 

mpl.rcParams['axes.linewidth'] = 0.5
mpl.rcParams['lines.linewidth'] = 1
mpl.rcParams["legend.numpoints"] = 1
mpl.rcParams["legend.frameon"] = False
#mpl.rcParams['figure.figsize'] = (4, 3)
mpl.rcParams['font.size'] = 10
mpl.rcParams['font.family'] = 'Arial'

# #Setting finally the Text characters to regular instead of italics.
mpl.rcParams['mathtext.rm'] = 'Arial'                 #'custom''Arial' #custom make mu italic, Arial straight
mpl.rcParams['mathtext.it'] = 'Arial: italic'         #make latex italics unless use \mathrm{}
mpl.rcParams['mathtext.default'] = 'it' #'default'
mpl.rcParams['xtick.direction'] = 'in'
mpl.rcParams['ytick.direction'] = 'in'
mpl.rcParams['xtick.major.width'] = 0.5
mpl.rcParams['ytick.major.width'] = 0.5
mpl.rcParams['xtick.major.size'] = 3.5 #default 3.5
mpl.rcParams['xtick.minor.size'] = 2 #default 2.0
mpl.rcParams['ytick.major.size'] = 3.5 #default 3.5
mpl.rcParams['ytick.minor.size'] = 2 #default 2.0

mpl.rcParams["errorbar.capsize"]= 3
mpl.rcParams['savefig.bbox'] = 'tight' # to avoid clipping mask cropping the axis label

## Setup figure
fig, ax = plt.subplots(figsize=(4,4))

def plt_percdens(fig, ax, n_p):
    dsx= calc_mob_dens(uuid[i], aspect_ratio=10, Vg=slice(-1759, None), Bfield=slice(-0.1, 0.1))    
    
    sigma_xx = dsx.sigma_xx * (h/e**2)
    n = dsx.n/1e11    
    
    ## Fit for percolation density based on fixed range 
    n_min = n.min()
    n_fit_range = 0.5
    n_min_idx = int(abs(n-n_min).argmin()) + 0
    n_max = n[n_min_idx] + n_fit_range
    n_max_idx = int(abs(n-n_max).argmin())    
    
    popt, pcov = curve_fit(percolation_function, n[n_min_idx:n_max_idx], sigma_xx[n_min_idx:n_max_idx], 
                           p0=(0.005,0), bounds=([0,0], [np.inf, n_max]), maxfev=100000)
    
    # x = np.linspace(n[n_min_idx], n[n_max_idx], 100)
    x = np.linspace(popt[1]+popt[1]*1e-3, np.max(n), 1000)
    ax.plot(n, np.log(sigma_xx), 'ko', ms=3, fillstyle='none')
    ax.plot(x, np.log(percolation_function(x, *popt)), '-r')
    ax.set_ylabel('log($\sigma_{\mathdefault{xx}})$ ($\mathdefault{e}^{2}/\mathdefault{h}$)')
    ax.set_xlabel('$p$ ($\mathdefault{10^{11}}$ $\mathdefault{cm}^{-2}$)', labelpad=1)
    ax.set_ylim([0,8])
    ax.set_yticks([0,2,4,6,8])
    ax.set_xticks([0.5, 1.0, 1.5])
    ax.text(0.96,4.85, '$\mathdefault{10^{10}}$', fontsize=8)
     
    n_p = n_p*10
    n_p_mean = np.mean(n_p)
    n_p_std = np.std(n_p)
    
    axin = ax.inset_axes([0.55, 0.1, 0.4, 0.5])
    axin.set_ylim([1.15, 1.30])
    x = np.linspace(0.1, 0.9, len(n_p))
    axin.plot(x, n_p, 'kd', ms=2)
    axin.errorbar((x.max()-x.min())/2+x.min(), n_p_mean, n_p_std, fmt='ko', ms=3, 
                  ecolor='k', elinewidth=1, capsize=2)
    axin.tick_params(labelsize=8)
    axin.set_ylabel('$p_{\mathdefault{p}}$ ($\mathdefault{cm}^{-2}$)', 
                    fontsize=8, labelpad=4, verticalalignment='center') 
    axin.set_xlim([0,1])
    axin.set_xticks([])
    axin.set_xticklabels([])  

plt_percdens(fig, ax, n_p)

